@@ -7,12 +7,13 @@ Debugger component used to debug each event in a system by printing
 each event to sys.stderr or to a Logger Component instnace.
 """
 
+import os
 import sys
 from cStringIO import StringIO
-from traceback import format_tb
 
+from handlers import handler
+from components import Component
 from circuits.tools import reprhandler
-from circuits.core import handler, Component
 
 class Debugger(Component):
     """Create a new Debugger Component
@@ -30,7 +31,8 @@ class Debugger(Component):
     IgnoreEvents = []
     IgnoreChannels = []
 
-    def __init__(self, errors=True, events=True, **kwargs):
+    def __init__(self, errors=True, events=True, file=None, logger=None,
+            **kwargs):
         "initializes x; see x.__class__.__doc__ for signature"
 
         super(Debugger, self).__init__()
@@ -38,37 +40,37 @@ class Debugger(Component):
         self.errors = errors
         self.events = events
 
-        self.logger = kwargs.get("logger", None)
-        self.file = kwargs.get("file", sys.stderr)
+        if type(file) is str:
+            self.file = open(os.path.abspath(os.path.expanduser(file)), "a")
+        elif type(file) is file or hasattr(file, "write"):
+            self.file = file
+        else:
+            self.file = sys.stderr
+
+        self.logger = logger
+
         self.IgnoreEvents.extend(kwargs.get("IgnoreEvents", []))
         self.IgnoreChannels.extend(kwargs.get("IgnoreChannels", []))
 
     @handler("exception", filter=True)
-    def exception(self, *args, **kwargs):
+    def exception(self, type, value, traceback, handler=None):
         if not self.errors:
             return
 
         s = StringIO()
 
-        if len(args) == 3:
-            type, value, traceback = args
-            handler = kwargs.get("handler", None)
-            if handler is None:
-                handler = "'Unknown'"
-            else:
-                handler = reprhandler(handler)
-            s.write("ERROR in %s (%s): %s\n" % (handler, type, value))
-            s.write("%s\n" % "".join(format_tb(traceback)))
+        if handler is None:
+            handler = ""
         else:
-            s.write("Unknown Error\n")
-            s.write("args:   %s\n" % repr(args))
-            s.write("kwargs: %s\n" % repr(kwargs))
+            handler = reprhandler(self.root, handler)
+
+        s.write("ERROR %s(%s): %s\n" % ("%s " % handler, type, value))
+        s.write("%s\n" % "".join(traceback))
 
         s.seek(0)
 
         if self.logger is not None:
-            for line in s:
-                self.logger.error(line)
+            self.logger.error(s.getvalue())
         else:
             self.file.write(s.read())
             self.file.flush()
@@ -87,6 +89,7 @@ class Debugger(Component):
             return
 
         channel = event.channel
+
         if True in [event.name == x.__name__ for x in self.IgnoreEvents]:
             return
         elif channel in self.IgnoreChannels:
