@@ -7,22 +7,23 @@
 This module implements WSGI Components.
 """
 
-import warnings
 from urllib import unquote
 from cStringIO import StringIO
-from traceback import format_exc
+from traceback import format_tb
+from sys import exc_info as _exc_info
 
-from circuits import handler, Component
+from circuits.core import handler, BaseComponent
 
 import wrappers
+from http import HTTP
+from events import Request
 from headers import Headers
-from utils import quoteHTML
+from errors import HTTPError
 from dispatchers import Dispatcher
-from events import Request, Response
-from errors import HTTPError, NotFound
-from constants import RESPONSES, DEFAULT_ERROR_MESSAGE
 
-class Application(Component):
+class Application(BaseComponent):
+
+    channel = "web"
 
     headerNames = {
             "HTTP_CGI_AUTHORIZATION": "Authorization",
@@ -35,6 +36,9 @@ class Application(Component):
     def __init__(self):
         super(Application, self).__init__()
 
+        self._finished = False
+
+        HTTP().register(self)
         Dispatcher().register(self)
 
     def translateHeaders(self, environ):
@@ -59,6 +63,7 @@ class Application(Component):
                 env("PATH_INFO", ""),
                 protocol,
                 env("QUERY_STRING", ""))
+        request.server = None
 
         request.remote = wrappers.Host(env("REMOTE_ADDR"), env("REMTOE_PORT"))
 
@@ -67,87 +72,46 @@ class Application(Component):
         request.wsgi_environ = environ
         request.body = env("wsgi.input")
 
-        response = wrappers.Response(None, request)
+        response = wrappers.Response(request)
         response.gzip = "gzip" in request.headers.get("Accept-Encoding", "")
 
         return request, response
 
-    def setError(self, response, status, message=None, traceback=None):
-        try:
-            short, long = RESPONSES[status]
-        except KeyError:
-            short, long = "???", "???"
-
-        if message is None:
-            message = short
-
-        explain = long
-
-        content = DEFAULT_ERROR_MESSAGE % {
-            "status": status,
-            "message": quoteHTML(message),
-            "traceback": traceback or ""}
-
-        response.body = content
-        response.status = "%s %s" % (status, message)
-
-    def _handleError(self, error):
-        response = error.response
-
-        v = self.send(error, "httperror", self.channel)
-
-        if v:
-            if issubclass(type(v), basestring):
-                response.body = v
-                res = Response(response)
-                self.send(res, "response", self.channel)
-            elif isinstance(v, HTTPError):
-                self.send(Response(v.response), "response", self.channel)
-            else:
-                assert v, "type(v) == %s" % type(v)
-
-    def response(self, response):
-        response.done = True
-
     def __call__(self, environ, start_response, exc_info=None):
-        while len(self) > 0:
-            self.flush()
+        self.request, self.response = self.getRequestResponse(environ)
+        self.push(Request(self.request, self.response))
 
-        request, response = self.getRequestResponse(environ)
+        self._finished = False
+        while not self._finished:
+            self.tick()
 
-        try:
-            req = Request(request, response)
+        self.response.prepare()
+        body = self.response.body
+        status = self.response.status
+        headers = self.response.headers.items()
 
-            v = self.send(req, "request", self.channel, errors=True)
+        start_response(status, headers, exc_info)
+        return body
 
-            if v:
-                if issubclass(type(v), basestring):
-                    response.body = v
-                    res = Response(response)
-                    self.send(res, "response", self.channel)
-                elif isinstance(v, HTTPError):
-                    self._handleError(v)
-                elif isinstance(v, wrappers.Response):
-                    res = Response(v)
-                    self.send(res, "response", self.channel)
-                else:
-                    assert v, "type(v) == %s" % type(v)
-            else:
-                error = NotFound(request, response)
-                self._handleError(error)
-        except:
-            error = HTTPError(request, response, 500, error=format_exc())
-            self._handleError(error)
-        finally:
-            body = response.process()
-            start_response(response.status, response.headers.items(), exc_info)
-            return [body]
+    @handler("response", filter=True, target="http")
+    def response(self, response):
+        self._finished = True
+        return True
 
-class Gateway(Component):
+class _Empty(str):
 
-    def __init__(self, app, path=""):
-        channel = None if not path else path
-        super(Gateway, self).__init__(channel=channel)
+    def __nonzero__(self):
+        return True
+
+empty = _Empty()
+del _Empty
+
+class Gateway(BaseComponent):
+
+    channel = "web"
+
+    def __init__(self, app, path="/"):
+        super(Gateway, self).__init__()
 
         self.app = app
         self.path = path
@@ -156,21 +120,24 @@ class Gateway(Component):
 
         self._request = self._response = None
 
-    def _createEnviron(self):
+        self.addHandler(self._on_request, "request", filter=True,
+                priority=len(path))
+
+    def createEnviron(self):
         environ = {}
         req = self._request
         env = environ.__setitem__
 
         env("REQUEST_METHOD", req.method)
         env("SERVER_NAME", req.host.split(":", 1)[0])
-        env("SERVER_PORT", "%i" % req.server.port)
+        env("SERVER_PORT", "%i" % (req.server.port or 0))
         env("SERVER_PROTOCOL", "HTTP/%d.%d" % req.server_protocol)
         env("QUERY_STRING", req.qs)
         env("SCRIPT_NAME", req.script_name)
         env("CONTENT_TYPE", req.headers.get("Content-Type", ""))
         env("CONTENT_LENGTH", req.headers.get("Content-Length", ""))
         env("REMOTE_ADDR", req.remote.ip)
-        env("REMOTE_PORT", "%i" % req.remote.port)
+        env("REMOTE_PORT", "%i" % (req.remote.port or 0))
         env("wsgi.version", (1, 0))
         env("wsgi.input", req.body)
         env("wsgi.errors", self._errors)
@@ -180,10 +147,10 @@ class Gateway(Component):
         env("wsgi.url_scheme", req.scheme)
 
         if req.path:
-            i = req.path.find(self.path) + len(self.path)
-            req.script_name, req.path = req.path[:i], req.path[i:]
+            req.script_name = req.path[:len(self.path)]
+            req.path = req.path[len(self.path):]
             env("SCRIPT_NAME", req.script_name)
-            env("PATH_INFO", unquote(req.path))
+            env("PATH_INFO", req.path)
 
         for k, v in req.headers.items():
             env("HTTP_%s" % k.upper().replace("-", "_"), v)
@@ -191,13 +158,23 @@ class Gateway(Component):
         return environ
 
     def start_response(self, status, headers, exc_info=None):
-        self._response.status = status
+        self._response.code = int(status.split(" ", 1)[0])
         for header in headers:
             self._response.headers.add_header(*header)
 
-    @handler("request", filter=True)
-    def request(self, request, response):
+    def _on_request(self, event, request, response):
+        if self.path and not request.path.startswith(self.path):
+            return
+
         self._request = request
         self._response = response
 
-        return "".join(self.app(self._createEnviron(), self.start_response))
+        try:
+            body = "".join(self.app(self.createEnviron(), self.start_response))
+            if not body:
+                body = empty
+            return body
+        except Exception, error:
+            etype, evalue, etraceback = _exc_info()
+            error = (etype, evalue, format_tb(etraceback))
+            return HTTPError(request, response, 500, error=error)
